/***
*
*	Copyright (C) 2002 The Wastes Project, All Rights Reserved.
*	
*	This product contains software technology from Valve Software, LLC,
*	Copyright  1996-2001, Valve LLC, All rights reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   The Wastes Project.  All other use, distribution, or modification is prohibited
*   without written permission from The Wastes Project.
*
***/
#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "weapons.h"
#include "weaponinfo.h"
#include "player.h"
#include "soundent.h"
#include "thewastes.h"
#include "game.h"
#include "gamerules.h"

// Entity Linkage
LINK_ENTITY_TO_CLASS(weapon_molotov,CMolotovCocktail);
LINK_ENTITY_TO_CLASS(weapon_fraggrenade,CFragGrenade);
LINK_ENTITY_TO_CLASS(weapon_pipebomb,CPipebomb);

enum explosivetype_e {
	ET_FRAG,
	ET_PIPEBOMB,
};

#ifndef CLIENT_DLL

// Baseline timed explosive class
class CThrownFrag : public CGrenade
{
public:
	void Spawn();

	static CThrownFrag *ThrowFrag( unsigned int usEvent, entvars_t *pevOwner, Vector vecStart, Vector vecVelocity, float time, char *pszExplosiveModel, float flDamage, float flResistance)
	{
		return ThrowExplosive( GetClassPtr((CThrownFrag*)NULL), usEvent, pevOwner, 
			                   vecStart, vecVelocity, time, 
							   pszExplosiveModel, flDamage, flResistance );
	}

	static CThrownFrag *ThrowExplosive(CThrownFrag *PtrFrag, unsigned int usEvent, entvars_t *pevOwner, Vector vecStart, Vector vecVelocity, float time, char *pszExplosiveModel, float flDamage, float flResistance);

	virtual void TraceAttack( entvars_t *pevAttacker, float flDamage, Vector vecDir, TraceResult *ptr, int bitsDamageType,int PenetrationValue = 0)
	{
		CBaseEntity *pEnt = CBaseEntity::Instance( pevAttacker );

		if( pEnt != NULL && pEnt->IsPlayer() )
		{
			if( flDamage > 20 && m_pfnThink == ExpThink )
			{
				SetThink( NULL );
				ExpDetonate();
			}
		}
	}

	// Callbacks
	void EXPORT ExpThink( void );
	void EXPORT ExpDetonate( void );
	void EXPORT ExpUse(CBaseEntity *pActivator,CBaseEntity *pCaller,USE_TYPE useType,float value);
private:
	unsigned int m_usFire1;
};
LINK_ENTITY_TO_CLASS(frag_grenade,CThrownFrag);

class CThrownPipeBomb : public CThrownFrag
{
public:
	void Spawn()
	{
		CThrownFrag::Spawn();
		pev->classname = MAKE_STRING("pipe_bomb");
	}

	static CThrownPipeBomb *ThrowPipeBomb( unsigned int usEvent, entvars_t *pevOwner, Vector vecStart, Vector vecVelocity, float time, char *pszExplosiveModel, float flDamage, float flResistance)
	{
		return (CThrownPipeBomb*)ThrowExplosive( GetClassPtr((CThrownPipeBomb*)NULL), usEvent, pevOwner, 
			                   vecStart, vecVelocity, time, 
							   pszExplosiveModel, flDamage, flResistance );
	}
};
LINK_ENTITY_TO_CLASS(pipe_bomb,CThrownPipeBomb);

#endif

/*************
	Posty's Liver Killer
*************/
#ifndef CLIENT_DLL

#define MOLOTOV_RADIUS    220	// Radius of effect
#define MOLOTOV_SMOKETIME 0.10f // Space of time between smoke puffs

class CMolotov : public CGrenade
{
public:
	void Spawn();
	void Precache();

	void EXPORT MolotovTouch(CBaseEntity *pOther);
	void EXPORT DamageThink( void );
	void EXPORT SmokeThink( void );

	static CMolotov *ShootMolotov(unsigned int usFireEvent,CBasePlayer *pPlayer,Vector vecStart,Vector vecVelocity);

	// Fire event
	unsigned int m_usFire1;
private:
	float m_flBurnTime;

	CBasePlayer *m_pPlayer;
};
LINK_ENTITY_TO_CLASS(molotov,CMolotov);

void CMolotov::Spawn()
{
	CGrenade::Spawn();

	m_flBurnTime = 0.0f;
	m_pPlayer    = NULL;

	pev->classname = MAKE_STRING("molotov");
}

void CMolotov::Precache()
{
	PRECACHE_MODEL("models/chromegibs.mdl");
}

void CMolotov::MolotovTouch(CBaseEntity *pOther)
{
	TraceResult tr;
	Vector      vecSpot;	// trace starts here!

	pev->enemy = pOther->edict();
	
	vecSpot = pev->origin - pev->velocity.Normalize() * 32;
	UTIL_TraceLine(vecSpot,vecSpot + pev->velocity.Normalize()*64,ignore_monsters,ENT(pev),&tr);

	// invisible
	pev->model      = iStringNull;
	pev->movetype   = MOVETYPE_NONE;
	pev->takedamage = DAMAGE_NO;
	pev->solid      = SOLID_NOT;

	// Pull out of the wall a bit
	if(tr.flFraction != 1.0)
		pev->origin = tr.vecEndPos + (tr.vecPlaneNormal * (pev->dmg-24) * 0.6f);

	pev->owner     = NULL;	// can't traceline attack owner if set
	pev->effects  |= EF_NODRAW;
	pev->velocity  = g_vecZero;
	pev->gravity   = 0.0f;	// Dont move

	SetTouch(NULL);

	int iContents = UTIL_PointContents(pev->origin);

	if(iContents != CONTENTS_WATER)
	{
		float flLifeTime = RANDOM_FLOAT(7.0f,10.0f);

		m_flBurnTime = gpGlobals->time + flLifeTime;

		// Initial damage (stronger)
		::RadiusDamage(pev->origin,pev,m_pPlayer->pev,110,MOLOTOV_RADIUS*1.25f,CLASS_NONE,DMG_BURN);

		// Radius Damage
		SetThink(DamageThink);
		pev->nextthink = gpGlobals->time + 0.25f;

		PLAYBACK_EVENT_FULL(FEV_GLOBAL,
							m_pPlayer->edict(),
							m_usFire1,
							0.0,
							(float*)&pev->origin,
							(float*)&tr.vecPlaneNormal, 
							flLifeTime,
							MOLOTOV_RADIUS, 
							0, 
							0, 
							0, 
							0);
	}
	else
	{
		// Molotovs dont do anything underwater.
		UTIL_Remove(this);
	}
}

void CMolotov::SmokeThink()
{
	MESSAGE_BEGIN(MSG_PVS,SVC_TEMPENTITY,pev->origin);
		WRITE_BYTE(TE_SMOKE);
		WRITE_COORD(pev->origin.x);
		WRITE_COORD(pev->origin.y);
		WRITE_COORD(pev->origin.z);
		WRITE_SHORT(g_sModelIndexSmoke);
		WRITE_BYTE((BYTE)0.20f);
		WRITE_BYTE(30); // framerate
	MESSAGE_END();

	// run again
	pev->nextthink = gpGlobals->time + MOLOTOV_SMOKETIME;
}


void CMolotov::DamageThink()
{
	// We have outlived our stay
	if(gpGlobals->time >= m_flBurnTime)
		UTIL_Remove(this);

	// Damage people as they come by
	::RadiusDamage(pev->origin,pev,m_pPlayer->pev,20,MOLOTOV_RADIUS,CLASS_NONE,DMG_BURN);

	// Do it again
	pev->nextthink = gpGlobals->time + 0.45f;
}

CMolotov *CMolotov::ShootMolotov(unsigned int usFireEvent,CBasePlayer *pPlayer,Vector vecStart,Vector vecVelocity)
{
	CMolotov *pRetMolotov = GetClassPtr((CMolotov*)NULL);

	pRetMolotov->m_usFire1 = usFireEvent;

	pRetMolotov->Spawn();
	// set the arc
	pRetMolotov->pev->gravity = 0.7f;
	
	// Set our owner
	pRetMolotov->m_pPlayer = pPlayer;

	UTIL_SetOrigin(pRetMolotov->pev,vecStart);
	pRetMolotov->pev->velocity = vecVelocity;
	pRetMolotov->pev->angles   = UTIL_VecToAngles(pRetMolotov->pev->velocity);
	pRetMolotov->pev->owner    = ENT(pPlayer->pev);
	
	// Tumble
	pRetMolotov->pev->avelocity.x = RANDOM_FLOAT(-100,-500);

	// Explode on contact
	pRetMolotov->SetTouch(MolotovTouch);
	pRetMolotov->pev->dmg = 50;

	// Smoke for us
	pRetMolotov->SetThink(SmokeThink);
	pRetMolotov->pev->nextthink = gpGlobals->time + MOLOTOV_SMOKETIME;

	SET_MODEL(ENT(pRetMolotov->pev),"models/w_molotov.mdl"); 

	return pRetMolotov;
}

#endif

// Player molotov control
void CMolotovCocktail::Spawn()
{
	Precache();

	pev->classname = MAKE_STRING("weapon_molotov");
	m_iId = WEAPON_MOLOTOVCOCKTAIL;

	m_flStartAttack = -1.0f;
	m_bJustThrown = FALSE;

	SET_MODEL(ENT(pev),"models/w_molotov.mdl");
	FallInit();
}

void CMolotovCocktail::Precache()
{
	PRECACHE_MODEL("models/v_molotov.mdl");
	PRECACHE_MODEL("models/p_molotov.mdl");
	PRECACHE_MODEL("models/w_molotov.mdl");

	m_usFire1 = PRECACHE_EVENT(1,"events/molotovcocktail.sc");

#ifndef CLIENT_DLL
	UTIL_PrecacheOther("molotov");
#endif
}

int CMolotovCocktail::GetItemInfo(ItemInfo *p)
{
	p->pszName   = STRING(pev->classname);
	p->iSlot     = 4;
	p->iPosition = iItemSlot() - 1;
	p->iFlags    = ITEM_FLAG_EXHAUSTIBLE|ITEM_FLAG_LIMITINWORLD;

	p->pszAmmo1  = "Molotov Cocktail";
	p->iMaxAmmo1 = AMMO_MAX_MOLOTOVCOCKTAIL;
	p->pszAmmo2  = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip  = WEAPON_NOCLIP;

	p->iId = m_iId = WEAPON_MOLOTOVCOCKTAIL;
	p->iWeight     = WEIGHT_MOLOTOVCOCKTAIL;

	return 1;
}

void CMolotovCocktail::AttachToPlayer(CBasePlayer *pPlayer)
{
	pPlayer->GiveAmmo(1,"Molotov Cocktail",AMMO_MAX_MOLOTOVCOCKTAIL);

	CWasteWeapon::AttachToPlayer(pPlayer);
}

int CMolotovCocktail::AddDuplicate(CBasePlayerItem *pItem)
{
	CBasePlayerWeapon *pWeapon = (CBasePlayerWeapon*)pItem;

	if(pWeapon != NULL)
		pWeapon->m_pPlayer->GiveAmmo(1,"Molotov Cocktail",AMMO_MAX_MOLOTOVCOCKTAIL);

	return 1;	// We just wanted to add ammo
}

// Not used
float CMolotovCocktail::flGetTiltedDamage()
{
	return 0;
}

char *CMolotovCocktail::szGetDeathNoticeString()
{
	return RANDOM_LONG(0,1) ? DEATH_MOLOTOVCOCKTAIL_1 : DEATH_MOLOTOVCOCKTAIL_2;
}

void CMolotovCocktail::ItemPostFrame()
{
	CWasteWeapon::ItemPostFrame();

	if( m_flNextPrimaryAttack <= UTIL_WeaponTimeBase() && m_bJustThrown )
	{
		m_bJustThrown = FALSE;
		SendWeaponAnim( MOLOTOV_DRAW, UseDecrement() ? 1 : 0 );
		m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 1.0f;
	}

	if(!(m_pPlayer->pev->button & IN_ATTACK))
		if(m_flStartAttack != -1.0f && m_flNextPrimaryAttack <= UTIL_WeaponTimeBase())
			ThrowMolotov();
}

void CMolotovCocktail::PrimaryAttack()
{
	if(m_flStartAttack != -1.0f)
		return;

	SendWeaponAnim(MOLOTOV_LIGHT,UseDecrement() ? 1 : 0);
	m_flStartAttack = gpGlobals->time;
	m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 1.0f;
}

void CMolotovCocktail::ThrowMolotov()
{
	SendWeaponAnim(MOLOTOV_THROW,UseDecrement() ? 1 : 0);

	m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

	m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.50f;
	m_flStartAttack = -1.0f;
	m_bJustThrown = TRUE;

#ifndef CLIENT_DLL
	float flVelocity = min(900,(gpGlobals->time - m_flStartAttack) * 350);

	CMolotov::ShootMolotov(m_usFire1,m_pPlayer,
						   m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs + gpGlobals->v_forward * 16,
	   				       gpGlobals->v_forward * flVelocity);

	if(m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
	{
		g_pGameRules->GetNextBestWeapon(m_pPlayer,this);
	}
#endif
}

BOOL CMolotovCocktail::Deploy()
{
	return DefaultDeploy("models/v_molotov.mdl","models/p_molotov.mdl",MOLOTOV_DRAW,"molotov");
}

BOOL CMolotovCocktail::CanHolster()
{
	// Can only holster if not ready to attack
	return (m_flStartAttack == -1.0f);
}

void CMolotovCocktail::Holster(int skiplocal)
{
	if( m_flStartAttack != -1.0f )
	{
		m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;
		m_flStartAttack = -1.0f;

#ifndef CLIENT_DLL
		CMolotov::ShootMolotov(m_usFire1,m_pPlayer,m_pPlayer->pev->origin,Vector(0,0,0));
#endif
	}

	if( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] < 1 )
	{
		// Remove this item
		m_pPlayer->pev->weapons &= ~(1<<WEAPON_MOLOTOVCOCKTAIL);
		SetThink(DestroyItem);
		pev->nextthink = gpGlobals->time + 0.1f;
	}

	CWasteWeapon::Holster(skiplocal);
}

/*************
	Base Timed Explosive Weapon
*************/
void CTimedExplosiveWeapon::Spawn()
{
	Precache();

	m_bFusePrimed = FALSE;
	m_bJustThrown = FALSE;
}

void CTimedExplosiveWeapon::Precache()
{
#ifndef CLIENT_DLL
	UTIL_PrecacheOther( "timed_explosive" );
#endif
}

int CTimedExplosiveWeapon::GetItemInfo( ItemInfo *p )
{
	p->pszName   = STRING(pev->classname);
	p->iSlot     = 4;
	p->iPosition = iItemSlot() - 1;
	p->iFlags    = ITEM_FLAG_EXHAUSTIBLE|ITEM_FLAG_LIMITINWORLD;

	return 1;
}

// Not used
float CTimedExplosiveWeapon::flGetTiltedDamage()
{
	return 0;
}

void CTimedExplosiveWeapon::ItemPostFrame()
{
	// We held the grenade too long! time to blow up in our hand
#ifndef CLIENT_DLL
	if(m_bFusePrimed && gpGlobals->time >= m_flStartAttack + m_flFuseTimer)
	{
		m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

		m_bFusePrimed   = FALSE;
		m_flStartAttack = -1.0f;

		if( m_iExplosiveType == ET_PIPEBOMB )
		{
			CThrownPipeBomb::ThrowPipeBomb( m_usExplosive, m_pPlayer->pev,m_pPlayer->pev->origin,Vector(0,0,0),0.0f,m_pszExplosiveModel,m_flDamage,m_flResistance);
		}	
		else if( m_iExplosiveType == ET_FRAG )
		{
			CThrownFrag::ThrowFrag( m_usExplosive, m_pPlayer->pev,m_pPlayer->pev->origin,Vector(0,0,0),0.0f,m_pszExplosiveModel,m_flDamage,m_flResistance);
		}

		if(m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		{
			g_pGameRules->GetNextBestWeapon(m_pPlayer,this);
		}
	}
#endif

	if(!(m_pPlayer->pev->button & (IN_ATTACK|IN_ATTACK2)))
	{
		if(m_flStartAttack != -1.0f && m_flNextPrimaryAttack <= UTIL_WeaponTimeBase())
		{
			if(!m_bFusePrimed)
				ThrowExplosive(m_flFuseTimer);
			else
				ThrowExplosive(m_flFuseTimer - (gpGlobals->time - m_flStartAttack));
		}
	}

	CWasteWeapon::ItemPostFrame();
}

void CTimedExplosiveWeapon::PrimaryAttack()
{
	if(m_flStartAttack != -1.0f)
		return;

	m_flStartAttack = gpGlobals->time;
	m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 0.75f;
}

void CTimedExplosiveWeapon::SecondaryAttack()
{
	if(m_flStartAttack != -1.0f)
		return;

	m_bFusePrimed = TRUE;
	
	m_flStartAttack = gpGlobals->time;
	m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1.0f;
}

void CTimedExplosiveWeapon::ThrowExplosive( float flFuse )
{
	m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;
	m_bJustThrown = TRUE;

	m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.75f;
	m_bFusePrimed         = FALSE;
	m_flStartAttack       = -1.0f;

#ifndef CLIENT_DLL
	float flVelocity = min(700,(gpGlobals->time - m_flStartAttack) * 250);

	if( m_iExplosiveType == ET_FRAG )
	{
		CThrownFrag::ThrowFrag( m_usExplosive, m_pPlayer->pev,
		             m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs + gpGlobals->v_forward * 16,
					 gpGlobals->v_forward * flVelocity,
					 flFuse,m_pszExplosiveModel,m_flDamage,m_flResistance );
	}
	else if( m_iExplosiveType == ET_PIPEBOMB )
	{
		CThrownPipeBomb::ThrowPipeBomb( m_usExplosive, m_pPlayer->pev,
		             m_pPlayer->pev->origin + m_pPlayer->pev->view_ofs + gpGlobals->v_forward * 16,
					 gpGlobals->v_forward * flVelocity,
					 flFuse,m_pszExplosiveModel,m_flDamage,m_flResistance );
	}

	if(m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
	{
		g_pGameRules->GetNextBestWeapon(m_pPlayer,this);
	}
#endif
}

BOOL CTimedExplosiveWeapon::CanHolster()
{
	// Can only holster if not ready to attack
	return (m_flStartAttack == -1.0f);
}

void CTimedExplosiveWeapon::Holster( int skiplocal )
{
	if( m_flStartAttack != -1.0f )
	{
		m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;

#ifndef CLIENT_DLL
		float flTime = (m_bFusePrimed == TRUE) ? m_flFuseTimer - (gpGlobals->time - m_flStartAttack) : m_flFuseTimer;

		if( m_iExplosiveType == ET_FRAG )
		{
			CThrownFrag::ThrowFrag(m_usExplosive, m_pPlayer->pev,
			             m_pPlayer->pev->origin,
						 Vector(0,0,0),
						 flTime,
						 m_pszExplosiveModel,m_flDamage,m_flResistance);
		}
		else if( m_iExplosiveType == ET_PIPEBOMB )
		{
			CThrownPipeBomb::ThrowPipeBomb(m_usExplosive, m_pPlayer->pev,
			             m_pPlayer->pev->origin,
						 Vector(0,0,0),
						 flTime,
						 m_pszExplosiveModel,m_flDamage,m_flResistance);
		}
#endif

		m_flStartAttack = -1.0f;
	}

	if( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] < 1 )
	{
		// Remove this item
		m_pPlayer->pev->weapons &= ~(1<<m_iId);
		SetThink(DestroyItem);
		pev->nextthink = gpGlobals->time + 0.1f;
	}
 
	CWasteWeapon::Holster(skiplocal);
}

/*************
	Frag Grenade
*************/
#define GRENADE_FUSE 3.0f

void CFragGrenade::Spawn()
{
	CTimedExplosiveWeapon::Spawn();

	pev->classname = MAKE_STRING("weapon_fraggrenade");
	m_iId = WEAPON_FRAGGRENADE;

	m_iExplosiveType    = ET_FRAG;
	m_flFuseTimer       = GRENADE_FUSE;
	m_flDamage          = 150.0f;
	m_flResistance      = 0.5f;
	m_pszExplosiveModel = "models/w_grenade.mdl";
	m_flStartAttack     = -1.0f;

	SET_MODEL(ENT(pev),"models/w_grenade.mdl");
	FallInit();
}

void CFragGrenade::Precache()
{
	PRECACHE_MODEL("models/v_grenade.mdl");
	PRECACHE_MODEL("models/p_grenade.mdl");
	PRECACHE_MODEL("models/w_grenade.mdl");

	m_usExplosive = PRECACHE_EVENT(1,"events/fraggrenade.sc");

	CTimedExplosiveWeapon::Precache();
}

int CFragGrenade::GetItemInfo(ItemInfo *p)
{
	p->pszAmmo1  = "FragGrenade";
	p->iMaxAmmo1 = AMMO_MAX_FRAGGRENADE;
	p->pszAmmo2  = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip  = WEAPON_NOCLIP;

	p->iId = m_iId = WEAPON_FRAGGRENADE;
	p->iWeight     = WEIGHT_FRAGGRENADE;

	return CTimedExplosiveWeapon::GetItemInfo(p);
}

void CFragGrenade::AttachToPlayer(CBasePlayer *pPlayer)
{
	pPlayer->GiveAmmo(1,"FragGrenade",AMMO_MAX_FRAGGRENADE);

	CWasteWeapon::AttachToPlayer(pPlayer);
}

int CFragGrenade::AddDuplicate(CBasePlayerItem *pItem)
{
	CBasePlayerWeapon *pWeapon = (CBasePlayerWeapon*)pItem;

	if(pWeapon != NULL)
		pWeapon->m_pPlayer->GiveAmmo(1,"FragGrenade",AMMO_MAX_FRAGGRENADE);

	return 1; // We just wanted to add ammo
}

char *CFragGrenade::szGetDeathNoticeString()
{
	return RANDOM_LONG(0,1) ? DEATH_FRAGGRENADE_1 : DEATH_FRAGGRENADE_2;
}

void CFragGrenade::PrimaryAttack()
{
	if( m_flStartAttack == -1.0f )
		SendWeaponAnim(FRAG_GOTOTHROW2,UseDecrement() ? 1 : 0);

	CTimedExplosiveWeapon::PrimaryAttack();
}

void CFragGrenade::SecondaryAttack()
{
	if( m_flStartAttack == -1.0f )
		SendWeaponAnim(FRAG_GOTOTHROW,UseDecrement() ? 1 : 0);

	CTimedExplosiveWeapon::SecondaryAttack();
}

void CFragGrenade::ItemPostFrame()
{
	if( m_flStartAttack == -1.0f && m_bJustThrown && 
		m_flNextPrimaryAttack <= UTIL_WeaponTimeBase() && 
		m_flNextSecondaryAttack <= UTIL_WeaponTimeBase() )
	{
		SendWeaponAnim( FRAG_DRAW, UseDecrement() ? 1 : 0 );
		m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1.0f;
		m_bJustThrown = FALSE;
	}

	CTimedExplosiveWeapon::ItemPostFrame();
}

void CFragGrenade::ThrowExplosive(float flFuse)
{
	CTimedExplosiveWeapon::ThrowExplosive( flFuse );
	SendWeaponAnim(FRAG_THROW,UseDecrement() ? 1 : 0);
}

BOOL CFragGrenade::Deploy()
{
	return DefaultDeploy("models/v_grenade.mdl","models/p_grenade.mdl",FRAG_DRAW,"frag");
}

/*************
	Pipebomb
*************/
#define PIPEBOMB_FUSE 4.0f

void CPipebomb::Spawn()
{
	CTimedExplosiveWeapon::Spawn();

	pev->classname = MAKE_STRING("weapon_pipebomb");
	m_iId = WEAPON_PIPEBOMB;

	m_iExplosiveType    = ET_PIPEBOMB;
	m_flFuseTimer       = PIPEBOMB_FUSE;
	m_flDamage          = 225.0f;
	m_flResistance      = 0.75f;
	m_pszExplosiveModel = "models/w_pipebomb.mdl";
	m_flStartAttack     = -1.0f;

	SET_MODEL(ENT(pev),"models/w_pipebomb.mdl");
	FallInit();
}

void CPipebomb::Precache()
{
	PRECACHE_MODEL("models/v_pipebomb.mdl");
	PRECACHE_MODEL("models/p_pipebomb.mdl");
	PRECACHE_MODEL("models/w_pipebomb.mdl");

	m_usExplosive = PRECACHE_EVENT(1,"events/pipebomb.sc");

	CTimedExplosiveWeapon::Precache();
}

int CPipebomb::GetItemInfo(ItemInfo *p)
{
	p->pszAmmo1  = "Pipebomb";
	p->iMaxAmmo1 = AMMO_MAX_PIPEBOMB;
	p->pszAmmo2  = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip  = WEAPON_NOCLIP;

	p->iId = m_iId = WEAPON_PIPEBOMB;
	p->iWeight     = WEIGHT_PIPEBOMB;

	return CTimedExplosiveWeapon::GetItemInfo(p);
}

void CPipebomb::AttachToPlayer(CBasePlayer *pPlayer)
{
	pPlayer->GiveAmmo(1,"Pipebomb",AMMO_MAX_PIPEBOMB);

	CWasteWeapon::AttachToPlayer(pPlayer);
}

int CPipebomb::AddDuplicate(CBasePlayerItem *pItem)
{
	CBasePlayerWeapon *pWeapon = (CBasePlayerWeapon*)pItem;

	if(pWeapon != NULL)
		pWeapon->m_pPlayer->GiveAmmo(1,"Pipebomb",AMMO_MAX_PIPEBOMB);

	return 1; // We just wanted to add ammo
}

char *CPipebomb::szGetDeathNoticeString()
{
	return RANDOM_LONG(0,1) ? DEATH_PIPEBOMB_1 : DEATH_PIPEBOMB_2;
}

void CPipebomb::PrimaryAttack()
{
	if( m_flStartAttack == -1.0f )
		SendWeaponAnim(PIPEBOMB_GOTOTHROW,UseDecrement() ? 1 : 0);

	CTimedExplosiveWeapon::PrimaryAttack();
}

void CPipebomb::SecondaryAttack()
{
	if( m_flStartAttack == -1.0f )
		SendWeaponAnim(PIPEBOMB_GOTOTHROW,UseDecrement() ? 1 : 0);

	CTimedExplosiveWeapon::SecondaryAttack();
}

void CPipebomb::ItemPostFrame()
{
	if( m_flStartAttack == -1.0f && m_bJustThrown && 
		m_flNextPrimaryAttack <= UTIL_WeaponTimeBase() && 
		m_flNextSecondaryAttack <= UTIL_WeaponTimeBase() )
	{
		SendWeaponAnim( PIPEBOMB_DRAW, UseDecrement() ? 1 : 0 );
		m_flNextPrimaryAttack = m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1.0f;
		m_bJustThrown = FALSE;
	}

	CTimedExplosiveWeapon::ItemPostFrame();
}

void CPipebomb::ThrowExplosive( float flFuse )
{
	CTimedExplosiveWeapon::ThrowExplosive( flFuse );
	SendWeaponAnim(PIPEBOMB_THROW,UseDecrement() ? 1 : 0);
}

BOOL CPipebomb::Deploy()
{
	return DefaultDeploy("models/v_pipebomb.mdl","models/p_pipebomb.mdl",PIPEBOMB_DRAW,"pipebomb");
}

/*************
	Explosive Timed Objects
	Used by frag grenade and pipebomb :)
*************/
#ifndef CLIENT_DLL

void CThrownFrag::Spawn()
{
	pev->classname = MAKE_STRING("frag_grenade");
	pev->movetype  = MOVETYPE_BOUNCE;
	pev->solid     = SOLID_BBOX;

	UTIL_SetSize(pev, Vector( 0, 0, 0), Vector(0, 0, 0));
}

CThrownFrag *CThrownFrag::ThrowExplosive(CThrownFrag *PtrFrag, unsigned int usEvent,entvars_t *pevOwner,Vector vecStart,Vector vecVelocity,float time,char *pszExplosiveModel,float flDamage, float flResistance)
{
	CThrownFrag *pRetExplosive = PtrFrag;

	pRetExplosive->Spawn();

	pRetExplosive->pev->dmg = flDamage;
	pRetExplosive->m_usFire1 = usEvent;

	UTIL_SetOrigin(pRetExplosive->pev,vecStart);

	pRetExplosive->pev->velocity = vecVelocity;
	pRetExplosive->pev->angles   = UTIL_VecToAngles(pRetExplosive->pev->velocity);
	pRetExplosive->pev->owner    = ENT(pevOwner);

	pRetExplosive->SetTouch(BounceTouch);	// Bounce if touched
	pRetExplosive->SetUse(ExpUse);

	// Take one second off of the desired detonation time and set the think to PreDetonate. PreDetonate
	// will insert a DANGER sound into the world sound list and delay detonation for one second so that 
	// the grenade explodes after the exact amount of time specified in the call to ShootTimed(). 

	pRetExplosive->pev->dmgtime = gpGlobals->time + time;

	pRetExplosive->SetThink(ExpThink);
	pRetExplosive->pev->nextthink = gpGlobals->time + 0.1;

	if(time < 0.1)
	{
		pRetExplosive->pev->nextthink = gpGlobals->time;
		pRetExplosive->pev->velocity = Vector( 0, 0, 0 );
	}

	// Tumble through the air
	pRetExplosive->pev->avelocity.x = -400;

	pRetExplosive->pev->gravity = flResistance;
	pRetExplosive->pev->friction = 0.8;

	SET_MODEL(ENT(pRetExplosive->pev),pszExplosiveModel);

	return pRetExplosive;
}

void CThrownFrag::ExpThink()
{
	if (!IsInWorld())
	{
		UTIL_Remove( this );
		return;
	}

	pev->nextthink = gpGlobals->time + 0.1;

	if (pev->dmgtime - 1 < gpGlobals->time)
	{
		CSoundEnt::InsertSound ( bits_SOUND_DANGER, pev->origin + pev->velocity * (pev->dmgtime - gpGlobals->time), 400, 0.1 );
	}

	if (pev->dmgtime <= gpGlobals->time)
	{
		SetThink( ExpDetonate );
	}
	if (pev->waterlevel != 0)
	{
		pev->velocity = pev->velocity * 0.5;
	}
}

void CThrownFrag::ExpDetonate()
{
	TraceResult tr;
	Vector		vecSpot;// trace starts here!

	vecSpot = pev->origin + Vector ( 0 , 0 , 8 );
	UTIL_TraceLine ( vecSpot, vecSpot + Vector ( 0, 0, -40 ),  ignore_monsters, ENT(pev), & tr);

	pev->model = iStringNull;//invisible
	pev->solid = SOLID_NOT;// intangible

	pev->takedamage = DAMAGE_NO;

	// Pull out of the wall a bit
	if ( tr.flFraction != 1.0 )
	{
		pev->origin = tr.vecEndPos + (tr.vecPlaneNormal * (pev->dmg - 24) * 0.6);
	}

	int iContents = UTIL_PointContents ( pev->origin );

	if(iContents == CONTENTS_WATER)
		pev->dmg /= 2;	// only half as effective underwater

	CSoundEnt::InsertSound ( bits_SOUND_COMBAT, pev->origin, NORMAL_EXPLOSION_VOLUME, 3.0 );
	entvars_t *pevOwner;
	if ( pev->owner )
		pevOwner = VARS( pev->owner );
	else
		pevOwner = NULL;

	pev->owner = NULL; // can't traceline attack owner if this is set

	RadiusDamage ( pev, pevOwner, pev->dmg, CLASS_NONE, DMG_BLAST );

	pev->effects |= EF_NODRAW;
	SetThink( SUB_Remove );
	pev->velocity = g_vecZero;
	pev->nextthink = gpGlobals->time + 0.3;

	PLAYBACK_EVENT_FULL(0,			// no longer FEV_GLOBAL (doesnt matter if its outside PVS/PAS, unlike the molotovs)
						ENT(pevOwner),
						m_usFire1,
						0.0,
						(float*)&pev->origin,
						(float*)&tr.vecPlaneNormal, 
						0,
						0, 
						(iContents == CONTENTS_WATER), 
						0, 
						0, 
						0);
}

void CThrownFrag::ExpUse(CBaseEntity *pActivator,CBaseEntity *pCaller,USE_TYPE useType,float value)
{
	if(pActivator->IsPlayer())
	{
		// TODO: The player should pick up this grenade :)
	}
}

#endif