/***
*
*	WastedFX Tool chain
*
*	Author: Joshua Coyne
*	Copyright (C) 2003 The Wastes Project, All Rights Reserved.
*
***/
#include "WastedFX.h"
#include "Window.h"
#include "GLWindow.h"
#include "Camera.h"
#include "Timer.h"

#define WINDOW_TITLE "Preview"

/*
=========================
 GLWndProc

 Callback procedures for GL Preview
=========================
*/
LRESULT CALLBACK GLWndProc( HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam )
{
	switch( iMsg )
	{
	case WM_SIZE:
		GetClassFromUserData<CGLWindow>( hWnd )->SetSize( LOWORD( lParam ), HIWORD( lParam ) );
		break;
	case WM_CLOSE:
		return 0;
	case WM_LBUTTONDOWN:
		GetClassFromUserData<CGLWindow>( hWnd )->OnLButtonDown();
		return 0;
	case WM_LBUTTONUP:
		GetClassFromUserData<CGLWindow>( hWnd )->OnLButtonUp();
		return 0;
	case WM_RBUTTONDOWN:
		GetClassFromUserData<CGLWindow>( hWnd )->OnRButtonDown();
		ShowCursor( FALSE );
		return 0;
	case WM_RBUTTONUP:
		GetClassFromUserData<CGLWindow>( hWnd )->OnRButtonUp();
		return 0;
	}

	return DefWindowProc( hWnd, iMsg, wParam, lParam );
}

CGLWindow::CGLWindow( char *pszClassname, HINSTANCE hInstance ) : CWindow( pszClassname, hInstance )
{
	m_pCamera = new CCamera;
	m_pTimer = new CTimer;

	if( RegisterWindow( CS_NOCLOSE|CS_OWNDC|CS_HREDRAW|CS_VREDRAW,
		                GLWndProc, 
						LoadIcon( NULL, IDI_WINLOGO ),
						LoadIcon( NULL, IDI_WINLOGO ),
						LoadCursor( NULL, IDC_ARROW ),
						(HBRUSH)GetStockObject( BLACK_BRUSH ),
						NULL ) )
	{
		HWND hParent = FindWindow( WINCLASS_MAIN, NULL );

		InstanceWindow( WINDOW_TITLE,
			            WS_CHILDWINDOW|WS_OVERLAPPEDWINDOW,
						hParent,
						10, 10, 512, 384 );	// TODO: Preferences!

		SetWindowLong( m_hWnd, GWL_USERDATA, (LONG)this );

		RECT rect;
		GetClientRect( m_hWnd, &rect );
		InitGL( rect.right, rect.bottom );

		ShowWindow( m_hWnd, SW_SHOW );
		UpdateWindow( m_hWnd );
	}
}

CGLWindow::~CGLWindow()
{
	delete m_pCamera;
	delete m_pTimer;

	if( m_hRC )
	{
		wglMakeCurrent( NULL, NULL );
		wglDeleteContext( m_hRC );
	}

	if( m_hDC )
		ReleaseDC( m_hWnd, m_hDC );
}

void CGLWindow::Update()
{
	m_pTimer->StartTimer();

	glClear( GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();

	// Set camera
	glRotatef( m_pCamera->m_Angles[0], 1, 0, 0 );
	glRotatef( m_pCamera->m_Angles[1], 0, 1, 0 );
	glRotatef( m_pCamera->m_Angles[2], 0, 0, 1 );
	glTranslatef( m_pCamera->m_Origin[0], m_pCamera->m_Origin[1], m_pCamera->m_Origin[2] );

	float x = 0.0f;
	float y = -5.0f;
	float height = 10.0f;
	float z = -50.0f;
	float width = 10.0f;

	glBegin( GL_QUADS );
		glColor4f( 0, 0, 0.5f, 1.0f );
		glVertex3f( -4096, -1500,  -4096 );
		glVertex3f( 4096, -1500, -4096  );
		glVertex3f( 4096, -1500, 4096  );
		glVertex3f( -4096 , -1500, 4096 );
	glEnd();

	glBegin(GL_TRIANGLES);		
		
		// These vertices create the Back Side
		glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);					// Top point
		glColor3ub(0, 255, 255); glVertex3f(x - width, y - height, z - width);	// Bottom left point
		glColor3ub(255, 0, 255); glVertex3f(x + width, y - height, z - width);  // Bottom right point

		// These vertices create the Front Side
		glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);					// Top point
		glColor3ub(0, 255, 255); glVertex3f(x + width, y - height, z + width);  // Bottom right point
		glColor3ub(255, 0, 255); glVertex3f(x - width, y - height, z + width);	// Bottom left point

		// These vertices create the Front Left Side
		glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);					// Top point
		glColor3ub(255, 0, 255); glVertex3f(x - width, y - height, z + width);	// Front bottom point
		glColor3ub(0, 255, 255); glVertex3f(x - width, y - height, z - width);	// Bottom back point

		// These vertices create the Front right Side
		glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);					// Top point
		glColor3ub(255, 0, 255); glVertex3f(x + width, y - height, z - width);	// Bottom back point
		glColor3ub(0, 255, 255); glVertex3f(x + width, y - height, z + width);	// Front bottom point
			
	glEnd();

	// Now render the bottom of our pyramid

	glBegin(GL_QUADS);

		// These vertices create the bottom of the pyramid
		glColor3ub(0, 0, 255); glVertex3f(x - width, y - height, z + width);	// Front left point
		glColor3ub(0, 0, 255); glVertex3f(x + width, y - height, z + width);    // Front right point
		glColor3ub(0, 0, 255); glVertex3f(x + width, y - height, z - width);    // Back right point
		glColor3ub(0, 0, 255); glVertex3f(x - width, y - height, z - width);    // Back left point
	glEnd();

	glBegin( GL_LINES );
		glColor4f( 1, 0, 0, 1 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 5, 0, 0 );

		glColor4f( 0, 1, 0, 1 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 0, 5, 0 );

		glColor4f( 0, 0, 1, 1 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 0, 0, 5 );
	glEnd();

	SwapBuffers( m_hDC );

	m_pTimer->EndTimer();

	// Camera input
	if( m_hWndParent == GetActiveWindow() )
		m_pCamera->ReadInput( m_pTimer->GetTime() );
	else
		m_pCamera->ResetInput();
}


void CGLWindow::OnLButtonDown()
{
	if( m_pCamera->m_iMouseMode == MOUSE_RIGHT )
		m_pCamera->m_iMouseMode = MOUSE_BOTH;
	else
		m_pCamera->m_iMouseMode = MOUSE_LEFT;
	ShowCursor( FALSE );
}

void CGLWindow::OnLButtonUp()
{
	if( m_pCamera->m_iMouseMode == MOUSE_BOTH )
		m_pCamera->m_iMouseMode = MOUSE_RIGHT;
	else
		m_pCamera->m_iMouseMode = MOUSE_NONE;
	ShowCursor( TRUE );
}

void CGLWindow::OnRButtonDown()
{
	if( m_pCamera->m_iMouseMode == MOUSE_LEFT )
		m_pCamera->m_iMouseMode = MOUSE_BOTH;
	else
		m_pCamera->m_iMouseMode = MOUSE_RIGHT;
}

void CGLWindow::OnRButtonUp()
{
	if( m_pCamera->m_iMouseMode == MOUSE_BOTH )
		m_pCamera->m_iMouseMode = MOUSE_LEFT;
	else
		m_pCamera->m_iMouseMode = MOUSE_NONE;
	ShowCursor( TRUE );
}

void CGLWindow::InitGL( int iWidth, int iHeight )
{
	psvec3_t origin = { 0.0f, 0.5f, -35.0f };
	psvec3_t angles = { 0.0f, 0.0f, 0.0f };

	m_hDC = GetDC( m_hWnd );

	if( !SetPixelFormat() )
		return;

	m_hRC = wglCreateContext( m_hDC );
	wglMakeCurrent( m_hDC, m_hRC );	

	memcpy( m_pCamera->m_Origin, &origin, sizeof( psvec3_t ) );
	memcpy( m_pCamera->m_Angles, &angles, sizeof( psvec3_t ) );
	SetSize( iWidth, iHeight );
}

BOOL CGLWindow::SetPixelFormat()
{
	PIXELFORMATDESCRIPTOR pfd = { 0 };
	int iPixelFormat;

	pfd.nSize = sizeof( PIXELFORMATDESCRIPTOR );
	pfd.nVersion = 1;

	pfd.dwFlags = PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER;	// TODO: Triple Buffer someday ?
	pfd.dwLayerMask = PFD_MAIN_PLANE;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 16;	// TODO: Set this ?
	pfd.cDepthBits = 16;
	pfd.cAccumBits = 0;
	pfd.cStencilBits = 0;	// Stencil ?

	if( (iPixelFormat = ChoosePixelFormat( m_hDC, &pfd )) == FALSE )
	{
		CWindow::Error( NULL, "PixelFormatError", "Unable to select proper pixel format", ERR_FATAL );
		return FALSE;
	}

	if( !::SetPixelFormat( m_hDC, iPixelFormat, &pfd ) )
	{
		CWindow::Error( NULL, "PixelFormatError", "Unable to set pixel format", ERR_FATAL );
		return FALSE;
	}

	return TRUE;
}

void CGLWindow::SetSize( int iWidth, int iHeight )
{
	if( iHeight == 0 )
		iHeight = 1;

	glViewport( 0, 0, iWidth, iHeight );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	// TODO: Set FOV ? Min/Max distances ?
	gluPerspective( 45.0f, (GLfloat)iWidth/(GLfloat)iHeight, 1.0f, 4096.0f );

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}
