/***
*
*	WastedFX Tool chain
*
*	Author: Joshua Coyne
*	Copyright (C) 2003 The Wastes Project, All Rights Reserved.
*
***/
#include "WastedFX.h"
#include "Window.h"

CWindow::CWindow( char *pszClassname, HINSTANCE hInstance ) :
	m_hWnd( NULL ),
	m_hWndParent( NULL ),
	m_hMenu( NULL )
{
	m_pszClassname = pszClassname;
	m_hInstance = hInstance;
}

CWindow::~CWindow()
{
	if( m_hMenu != NULL )
		DestroyMenu( m_hMenu );

	UnregisterClass( m_pszClassname, m_hInstance );
}

void CWindow::Update()
{
}

BOOL CWindow::RegisterWindow( int style, void *WndProc,
							  HICON hIcon, HICON hIconSm,
							  HCURSOR hCursor,
							  HBRUSH hBrush,
							  HMENU hMenu )
{
	WNDCLASSEX wndClass;

	wndClass.cbSize = sizeof( wndClass );
	wndClass.hInstance = m_hInstance;
	wndClass.style = style;
	wndClass.lpfnWndProc = (WNDPROC)WndProc;
	wndClass.cbClsExtra = 0;
	wndClass.cbWndExtra = 0;
	wndClass.hIcon = hIcon;
	wndClass.hIconSm = hIconSm;
	wndClass.hCursor = hCursor;
	wndClass.hbrBackground = hBrush;
	wndClass.lpszMenuName = NULL;
	wndClass.lpszClassName = m_pszClassname;

	if( !RegisterClassEx( &wndClass ) )
		return FALSE;

	m_hMenu = hMenu;

	return TRUE;
}

BOOL CWindow::InstanceWindow( char *pszTitle, int style, HWND hWndParent,
					          int x, int y, int w, int h )
{
	m_hWnd = ::CreateWindow( m_pszClassname, pszTitle, style,
		                     x, y, w, h, hWndParent, m_hMenu, m_hInstance,
							 NULL );

	m_hWndParent = hWndParent;

	return TRUE;
}

void CWindow::Error( CWindow *pWindow, char *pszCaption, char *pszMsg, int iErrMode )
{
	HWND hWnd = NULL;

	if( pWindow != NULL )
		hWnd = pWindow->m_hWnd;

	switch( iErrMode )
	{
	case ERR_ALERT:
		MessageBox( hWnd, pszMsg, pszCaption, MB_OK|MB_ICONEXCLAMATION );
		break;
	case ERR_INFO:
		MessageBox( hWnd, pszMsg, pszCaption, MB_OK|MB_ICONINFORMATION );
		break;
	case ERR_FATAL:
		MessageBox( hWnd, pszMsg, pszCaption, MB_OK|MB_ICONERROR );
		PostQuitMessage( 0 );
		break;
	}
}
